;**********************************************************************
;                                                                     *
;    Filename:	    enc-parallel.asm                                  *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        Peter Jakab <el@jap.hu>                           *
;                   http://jap.hu/electronic/                         *
;**********************************************************************
;NOTES
;encoder v4.2
;
;**********************************************************************
;HISTORY
;
; 050-20091212 16-bit parallel encoder for 16f57
; 051-20091212 (use mtxv4.035)
; 052-20091212 add modulation (use mtxv4.036)
;
;**********************************************************************
	list      p=16f57


#include <p16f5x.inc>
#include <mtxv4.inc>

MOD_EN	EQU 1 ; modulation enable pin on PORTA
TX_EN	EQU 2 ; tx enable pin on PORTA
TX_OUT	EQU 0

	__CONFIG   _CP_OFF & _WDT_OFF & _XT_OSC

mainda		UDATA
rcnt		res 1
tmp		res 1

vectors		CODE 0
  		goto    main		; go to beginning of program

prog		CODE 5

name		dt 0x51, "enc-051.asm jap20091212", 0
en_mask_portb	dt b'00000000' ; enable mask for PB bits (0=enable)
en_mask_portc	dt b'11111111' ; enable mask for PC bits (0=enable)

main
		;HARDWARE INIT

		movlw (1<<MOD_EN) | (1<<TX_EN)
		tris PORTA
		movlw 0xff
		tris PORTB
		tris PORTC

		movlw 0
		movwf PORTA
		movwf PORTB
		movwf PORTC

		; clrwdt
		; movlw (0<<T0CS) | (1<<PSA) | 0x07 ; prescaler to WDT, 1:128
		; option

		call mtx_init

loop		call trigger_tx
		btfsc STATUS, Z ; wait for TX_EN to become low
		goto loop

		; set modulation type
		movf PORTA, W
		andlw (1<<MOD_EN)
		call mtx_setflag

loop2		movlw 3
		movwf rcnt

loop3		movf PORTB, W
		movwf mtx_buffer
		movf PORTC, W
		movwf mtx_buffer+1
		call mtx_send

		call trigger_tx
		btfss STATUS, Z ; if TX_EN is still low, goto loop2
		goto loop2
				; TX_EN became high
		decfsz rcnt, F
		goto loop3

		goto loop

trigger_tx	; test if we should transmit or no
		; if yes, return Z=0 (transmit)

		; test TX_EN input.
		movlw (1<<TX_EN)
		andwf PORTA, W
		xorlw (1<<TX_EN)
		btfss STATUS, Z
		return ; yes, Z=0

		; test for selected pins
		call en_mask_portb
		iorwf PORTB, W
		xorlw 0xff
		btfss STATUS, Z
		return ; yes, Z=0

		; test for selected pins
		call en_mask_portc
		iorwf PORTC, W
		xorlw 0xff
		return ; if yes, Z=0

		end

