;**********************************************************************
;                                                                     *
;    Filename:	    mtxv4.asm                                         *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;    Www page:      http://jap.hu/electronic/                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;    Notes:
;
; RF transmitter prot.v4.2 (Manchester)
;**********************************************************************
;
; header : 20xbit1, 1xbit0
; bytes : 8xbitX, 1xbit1, 1xbit0
;
; packet structure: <data0> <data1> <data2> <checksum>
;
; 020 - library version
; untested!
;
; 021 - add half frame variable
;       remove buffer initialization
;
; 030-20050102 codec-v4.2: 
;              checksum with CRC-8
;              T=350 usec
;              3-byte buffer length
;
; 035-20091212 1 stack level version for lowend PIC devices
; 036-20091213 add MOD capability
; 037-20091213 optimize MOD bit1->bit0 transitions - jitterfree
;**********************************************************************

;define TXINV
;invert TX output so inactive is HIGH?

	list      p=16F57
	#include <p16F5x.inc>

txbit	EQU 0x01
txport	EQU PORTA

#ifndef TXINV
#define TXLOW 0
#define TXHIGH txbit
#else
#define TXLOW txbit
#define TXHIGH 0
#endif

	GLOBAL mtx_buffer, mtx_flags, mtx_init, mtx_send, mtx_delay, mtx_setflag

packet_len	EQU 2

;***** VARIABLE DEFINITIONS
mtxdata		UDATA

count1		res 1
count2		res 1
ncnt		res 1
bt		res 1
sum		res 1
mtx_buffer	res 2

mtx_delay	res 1 ; half_frame delay
mtx_modcnt	res 1 ; modulation cycle count
mtx_flags	res 1
FLAG_MOD	EQU 1

;**********************************************************************

mtx		CODE

mtx_init	movlw .117 ; 350 usec
		movwf mtx_delay
		movlw .13 ; 350/26 us: cycles of 38.4kHz
		movwf mtx_modcnt
		clrf mtx_flags
		return
		;
mtx_setflag	movwf mtx_flags
		return

mtx_send
		; send out buffer
outbuf		movlw 0x13 ; 20xbit1, 1xbit0

header		movwf count2
head0		call bit1
		goto $+1
		goto $+1
		movlw TXLOW
		btfsc mtx_flags, FLAG_MOD
		movwf txport

		decfsz count2,F
		goto head0

		call bit10

		movlw mtx_buffer
		movwf FSR
		movlw packet_len
		movwf count1
		movlw 0xff
		movwf sum
		;
outbu0		movf INDF,W
		call update_sum
		movf INDF,W

outbytea	movwf bt
		movlw 8
		movwf count2
outby0a		rlf bt,F

		movlw TXLOW
		btfsc mtx_flags, FLAG_MOD
		movwf txport

		btfsc STATUS,C
		goto outby1a
		call bit0
		goto outby2a
outby1a		call bit1
outby2a		decfsz count2,F
		goto outby0a
		;
		call bit10

		incf FSR,F
		decfsz count1,F
		goto outbu0
		movf sum,W

outbyteb	movwf bt
		movlw 8
		movwf count2
outby0b		rlf bt,F

		movlw TXLOW
		btfsc mtx_flags, FLAG_MOD
		movwf txport

		btfsc STATUS,C
		goto outby1b
		call bit0
		goto outby2b
outby1b		call bit1
outby2b		decfsz count2,F
		goto outby0b
		;
		call bit10

		; buffer is sent

		return

update_sum	; fast CRC-8 algorithm with poly x^8+x^5+x^4+1
		; executes in 23 cycles per update

		xorwf	sum,f
		clrw
		btfsc	sum,7
		xorlw	0x7a
		btfsc	sum,6
		xorlw	0x3d
		btfsc	sum,5
		xorlw	0x86
		btfsc	sum,4
		xorlw	0x43
		btfsc	sum,3
		xorlw	0xb9
		btfsc	sum,2
		xorlw	0xc4
		btfsc	sum,1
		xorlw	0x62
		btfsc	sum,0
		xorlw	0x31
		movwf	sum
		return
		;

bit0		btfsc mtx_flags, FLAG_MOD
		goto bit0_mod
bit0_nomod	movlw TXHIGH ; HIGH
		movwf txport

ndelaya0	movf mtx_delay, W
		movwf ncnt
ndelaya1	decfsz ncnt, F
		goto ndelaya1

		movlw TXLOW ; to LOW transition
		movwf txport

ndelayb0	movf mtx_delay, W
		movwf ncnt
ndelayb1	decfsz ncnt, F
		goto ndelayb1

		return

bit0_mod	movf mtx_modcnt, W
		movwf ncnt
		goto $+1
bit0_cycle	movlw TXHIGH
		movwf txport
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		nop
		movlw TXLOW
		movwf txport
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		decfsz ncnt, F
		goto bit0_cycle

ndelayb2	movf mtx_delay, W
		movwf ncnt
ndelayb3	decfsz ncnt, F
		goto ndelayb3

		return

bit1		btfsc mtx_flags, FLAG_MOD
		goto bit1_mod
bit1_nomod	movlw TXLOW ; LOW
		movwf txport

ndelayc0	movf mtx_delay, W
		movwf ncnt
ndelayc1	decfsz ncnt, F
		goto ndelayc1

		movlw TXHIGH ; to HIGH transition
		movwf txport

ndelaye0	movf mtx_delay, W
		movwf ncnt
ndelaye1	decfsz ncnt, F
		goto ndelaye1

		return

bit1_mod	movlw TXLOW
		movwf txport
ndelaye2	movf mtx_delay, W
		movwf ncnt
ndelaye3	decfsz ncnt, F
		goto ndelaye3

		movf mtx_modcnt, W
		movwf ncnt
		decf ncnt, F
bit1_cycle	movlw TXHIGH
		movwf txport
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		nop
		movlw TXLOW
		movwf txport
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		decfsz ncnt, F
		goto bit1_cycle

		; last cycle
		nop
		movlw TXHIGH
		movwf txport
		goto $+1
		goto $+1
		return

		goto $+1
		nop
		movlw TXLOW
		movwf txport
		return

bit10		btfss mtx_flags, FLAG_MOD
		goto bit10_nomod

		movlw TXLOW
		movwf txport
ndelayf2	movf mtx_delay, W
		movwf ncnt
ndelayf3	decfsz ncnt, F
		goto ndelayf3

		movf mtx_modcnt, W
		movwf ncnt
bit1n_cycle	movlw TXHIGH
		movwf txport
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		goto $+1
		nop
		movlw TXLOW
		movwf txport
		goto $+1
		decfsz ncnt, F
		goto bit1n_4
		nop
		goto bit0_mod

bit1n_4		goto $+1
		goto $+1
		goto bit1n_cycle

bit10_nomod	movlw TXLOW ; LOW
		movwf txport

ndelayc4	movf mtx_delay, W
		movwf ncnt
ndelayc5	decfsz ncnt, F
		goto ndelayc5

		movlw TXHIGH ; to HIGH transition
		movwf txport

ndelaye4	movf mtx_delay, W
		movwf ncnt
ndelaye5	decfsz ncnt, F
		goto ndelaye5

		goto bit0_nomod


		end

